#ifndef LIBSW_DRIVER_H
#define LIBSW_DRIVER_H

#include "libsw_types.h"

typedef struct
{
  char *name;
  int (*init)(void **data, libsw_hwprm_t *hwprm);
  int (*free)(void *data);

  int (*connect_hardware)(void *h);
  int (*init_hardware_common)(void *h);
  int (*reset_hardware)(void *h, uint16_t level);

  int (*get_hw_info)(void *h, hw_info_t *info);

  libsw_obj_t *(*get_obj_by_name)(void *h, libsw_objid_t id, const char *name);
  libsw_obj_t *(*get_obj_by_num)(void *h, libsw_objid_t id, uint16_t num);
  libsw_obj_t *(*get_obj_by_idx)(void  *h, libsw_objid_t id, uint16_t idx);
  uint16_t (*get_obj_index)(void *h, const libsw_obj_t *);
  uint16_t (*get_obj_count)(void *h, libsw_objid_t id);
  uint16_t (*set_obj_name)(void *h, libsw_obj_t *, const char *name);
  uint16_t (*set_obj_flags)(void *h, libsw_obj_t *, uint32_t flags, uint32_t mask);
  uint16_t (*get_obj_flags)(void *h, const libsw_obj_t *obj, uint32_t *flags);
  int (*get_obj_stat)(void  *h, const libsw_obj_t *, stat_item_t *, uint16_t *cnt);
  int (*clear_obj_stat)(void  *h, const libsw_obj_t *);

  int (*get_obj_status)(void *h, const libsw_obj_t *obj, iface_status_t *);
  int (*set_obj_enable)(void *h, const libsw_obj_t *obj, uint8_t enable);
  int (*set_iface_loopback)(void *h, const iface_desc_t *, uint8_t enable);

  int (*get_iface_portsecurity)(void *h, const iface_desc_t *, iface_portsec_t *);
  int (*set_iface_portsecurity)(void *h, const iface_desc_t *, uint8_t value);

  int (*get_obj_dynamic_learning)(void *h, const libsw_obj_t *, uint8_t *enable);
  int (*set_obj_dynamic_learning)(void *h, const libsw_obj_t *, uint8_t enable);

  int (*get_obj_defaultvlan)(void *h, const libsw_obj_t *, vlan_desc_t **vlan);
  int (*set_obj_defaultvlan)(void *h, const libsw_obj_t *, vlan_desc_t *vlan);

  int (*get_obj_vlans)(void *h, const libsw_obj_t *, iface_vlan_t *, uint16_t *count);

  int (*add_obj_vlan)(void *h, const libsw_obj_t *, iface_vlan_t *);
  int (*del_obj_vlan)(void *h, const libsw_obj_t *, vlan_desc_t *vlan);

  int (*get_vlan_members) ( void *h, const vlan_desc_t *vlan,
                            vlan_member_t *members, uint16_t *count,
                            libsw_vlan_member_filter_t filter );

  int (*set_iface_vmt)(void *h, const iface_desc_t *iface, vmt_desc_t *);
  int (*get_iface_vmt)(void *h, const iface_desc_t *iface, vmt_desc_t **);
  int (*get_vmt)(void *h, const vmt_desc_t *, vmt_item_t *, uint16_t *count);
  int (*add_vmt_item)(void *h, const vmt_desc_t *, vmt_item_t *item);
  int (*del_vmt_item)(void *h, const vmt_desc_t *, vmt_item_t *item);

  uint16_t (*get_update_time)(void *h);
  int (*update)(void *h);

  int (*get_obj_speed_mode)(void *h, const libsw_obj_t *, speed_mode_t *mode);
  int (*set_iface_speed_mode)(void *h, const iface_desc_t *, speed_mode_t *mode);

  int (*get_iface_lag_state)(void *h, const iface_desc_t *, uint8_t *);
  int (*set_iface_lag_state)(void *h, const iface_desc_t *, uint8_t );
  int (*get_iface_lag)(void *h, const iface_desc_t *, lag_desc_t **lag);
  int (*set_iface_lag)(void *h, const iface_desc_t *, const lag_desc_t *lag);
  int (*get_lag_distribution)(void *h, const lag_desc_t *lag, uint16_t *mode);
  int (*set_lag_distribution)(void *h, const lag_desc_t *lag, uint16_t mode);
  int (*get_lag_members)(void *h, const lag_desc_t *lag, iface_desc_t *ifs[], uint16_t *count);

  int (*get_portmirror)(void *h, const iface_desc_t *, iface_desc_t **);
  int (*set_portmirror)(void *h, const iface_desc_t *, const iface_desc_t *);

  int (*get_vlanmirror)(void *h, const vlan_desc_t *, iface_desc_t **);
  int (*set_vlanmirror)(void *h, const vlan_desc_t *, const iface_desc_t *);

  int (*get_sfp_status)(void *h, const iface_desc_t *iface, uint8_t *installed);
  int (*read_sfp_regs)(void *h, const iface_desc_t *, uint8_t bank, uint16_t start,
                       uint16_t count, void *data);
  int (*handle_after_sfp_install)(void *h, const iface_desc_t *);

  int (*get_mac_table)(void *h, mac_table_t *, uint32_t *count);
  int (*add_mac_table_entry)(void *h, const mac_table_t *);
  int (*del_mac_table_entry)(void *h, const mac_table_t *);

  int (*get_obj_stp)(void *h, const libsw_obj_t *obj, const vlan_desc_t *vlan, uint8_t *en);
  int (*set_obj_stp)(void *h, const libsw_obj_t *obj, const vlan_desc_t *vlan, uint8_t en);
  int (*get_stp)(void *h, uint8_t *en);
  int (*set_stp)(void *h, uint8_t en);
  int (*get_stp_mode)(void *h, stp_mode_t *stpmode);
  int (*set_stp_mode)(void *h, stp_mode_t stpmode);
  int (*get_obj_stp_state)(void *h, const libsw_obj_t *obj, const vlan_desc_t *, stp_state_t *stpstate);
  int (*set_obj_stp_state)(void *h, const libsw_obj_t *obj, const vlan_desc_t *, stp_state_t stpstate);
  int (*get_obj_stp_params)(void *h, const libsw_obj_t *obj, const vlan_desc_t *vlan, libsw_iface_stp_params_t *p);
  int (*set_obj_stp_params)(void *h, const libsw_obj_t *obj, const vlan_desc_t *vlan, const libsw_iface_stp_params_t *p);
  int (*get_stp_params)(void *h, const vlan_desc_t *vlan, libsw_stp_params_t *p);
  int (*set_stp_params)(void *h, const vlan_desc_t *vlan, const libsw_stp_params_t *p);
  int (*get_stp_status)(void *h, const vlan_desc_t *vlan, libsw_stp_status_t *p);
  int (*set_stp_status)(void *h, const vlan_desc_t *vlan, const libsw_stp_status_t *p);
  int (*get_obj_stp_status)(void *h, const libsw_obj_t *obj, const vlan_desc_t *vlan, libsw_iface_stp_status_t *p);
  int (*set_obj_stp_status)(void *h, const libsw_obj_t *obj, const vlan_desc_t *vlan, const libsw_iface_stp_status_t *p);

  int (*set_igmp_snooping_enable)(void *h, const vlan_desc_t *vlan, uint8_t en);
  int (*get_igmp_snooping_enable)(void *h, const vlan_desc_t *vlan, uint8_t *en);
  int (*set_obj_igmp_snooping_enable)(void *h, const libsw_obj_t *obj,
                                        const vlan_desc_t *vlan, uint8_t en);
  int (*get_obj_igmp_snooping_enable)(void *h, const libsw_obj_t *obj,
                                        const vlan_desc_t *vlan, uint8_t *en);
  int (*set_igmp_snooping_params)(void *h, const vlan_desc_t *vlan,
                                  const libsw_igmp_snooping_params_t *prms);
  int (*get_igmp_snooping_params)(void *h, const vlan_desc_t *vlan,
                                  libsw_igmp_snooping_params_t *prms);
  int (*set_obj_igmp_snooping_params)(void *h, const libsw_obj_t *obj,
                                        vlan_desc_t *vlan, 
                                        const libsw_iface_igmp_snooping_params_t *p);
  int (*get_obj_igmp_snooping_params)(void *h, const libsw_obj_t *obj,
                                        const vlan_desc_t *vlan, 
                                        libsw_iface_igmp_snooping_params_t *p);

  int (*acl_add_rule)(void *h, libsw_acl_condition_list_t *rl, libsw_acl_action_t *);
  int (*acl_del_rule)(void *h, libsw_acl_condition_list_t *rl, libsw_acl_action_t *);
  int (*acl_del_rule_by_id)(void *h, uint16_t ruleid);
  int (*acl_get_rules)(void *h, libsw_acl_rule_list_t *rsl);
  int (*acl_clear_all)(void *h);

  int (*set_mac_aging_time)(void *h, uint32_t time);
  int (*get_mac_aging_time)(void *h, uint32_t *time);

  int (*change_ip_iface)(void *h, const libsw_ipiface_t *ipif, libsw_change_action_t act);
  int (*get_ip_ifaces)(void *h, libsw_ipiface_t *ifs, uint16_t *count);

  int (*change_ip_route)(void *h, const vlan_desc_t *vlan, const libsw_iproute_t *rt, libsw_change_action_t act);
  int (*get_ip_routes)(void *h, const vlan_desc_t *vlan, libsw_iproute_t *rts, uint16_t *count);

  int (*open_slow_protocol_fd)(void *h, int *fd, libsw_slow_prot_id_t *sp);
  int (*read_slow_protocol_packet)(void *h, int fd, uint8_t flags, char *buf, size_t *size, libsw_slow_prot_hdr_t *hdr);
  int (*prepare_packet_for_send) ( void *h,
                                   const libsw_obj_t *obj,
                                   const vlan_desc_t *vlan,
                                   void *data,
                                   size_t *len,
                                   size_t bufsize );
  int (*send_packet) ( void *h,
                       const libsw_obj_t *obj,
                       const void *prepared_data,
                       size_t size );
  int (*set_management_configuration_ready)( void *h, char ready );
  int (*check_jumbo_available)( void *h );

  int (*set_port_jumbo_frame_length)( void *h, const iface_desc_t *iface, uint16_t len );
  int (*disable_jumbo_frames_support)( void *h, const iface_desc_t *iface );
  int (*get_iface_frame_length)( void *h, const iface_desc_t *iface, uint16_t *len );

} switch_hw_t;

#endif
