#ifndef LIBSW_TYPES_H
#define LIBSW_TYPES_H

#include "libdb.h"

typedef struct
{
  void *sfp;
  dbg_desc_t *log;
  libsw_db_api_t *dbapi;
} libsw_hwprm_t;

#define IFACE_STS_LINK_UP          0x01
#define IFACE_STS_ENABLED          0x02
#define IFACE_STS_LOOPBACK_ENABLED 0x04
#define IFACE_STS_LA_ENABLED       0x08
#define IFACE_STS_STP_ENABLED      0x10
#define IFACE_STS_VLAN_STP_ENABLED 0x20

#define PORT_SECURITY_ENABLED  1

#define PORT_SECURITY_MODE_UNKNOWN  1
#define PORT_SECURITY_MODE_SHUTDOWN 2
#define PORT_SECURITY_MODE_RESTRICT 3

#define LIBSW_IFACE_SPEED_MODE_UNKNOWN  0
#define LIBSW_IFACE_SPEED_MODE_1G  1000
#define LIBSW_IFACE_SPEED_MODE_10G 10000
#define LIBSW_IFACE_SPEED_MODE_20G 20000
#define LIBSW_IFACE_SPEED_MODE_RSPIN_DOWN    0
#define LIBSW_IFACE_SPEED_MODE_RSPIN_UP      1
#define LIBSW_IFACE_SPEED_MODE_RSPIN_DEFAULT 2

#define LIBSW_RSTLEV_PCPU 2
#define LIBSW_RSTLEV_MCPU 4

#define LIBSW_SPFL_REMOVE_80211TAG 1
#define LIBSW_SPFL_ADD_80211TAG    2

/* Error codes */
#define LIBSW_OK                0
#define LIBSW_ERROR            -1
#define LIBSW_NOT_FOUND        -2
#define LIBSW_NOT_SUPPORTED    -3
#define LIBSW_NOT_INSTALLED    -4
#define LIBSW_NOT_ALLOWED      -5
#define LIBSW_WRONG_OBJECT     -6

#define LIBSW_OBJNAMESIZE 16
#define LIBSW_OBJRANGESIZE 128
#define LIBSW_MAXVLANID 4095

typedef struct
{
  uint8_t flags;
  uint8_t speed;
} iface_portsec_t;

#define LIBSW_IFTYPE_NONE       0
#define LIBSW_IFTYPE_SFPP       1
#define LIBSW_IFTYPE_SFP        2
#define LIBSW_IFTYPE_1000BASE   3
#define LIBSW_IFTYPE_100BASE    4

#define LIBSW_OBJFLAG_HIDDEN    1
#define LIBSW_OBJFLAG_ACTIVE    2
#define LIBSW_OBJFLAG_LOCKED    4

/*
#define LIBSW_IFFLAG_HIDEN        1
#define LIBSW_VLANFLAG_ACTIVE     2
*/

#define LIBSW_LAG_DIST_DMAC  0
#define LIBSW_LAG_DIST_SMAC  1
#define LIBSW_LAG_DIST_MAC   2
#define LIBSW_LAG_DIST_RPORT 3
#define LIBSW_LAG_DIST_IP    4
#define LIBSW_LAG_DIST_VLAN  5
#define LIBSW_LAG_DIST_SIP   6
#define LIBSW_LAG_DIST_DIP   7

#define LIBSW_SFP_DETAILS_FULL 1


typedef enum {STPMODE_DISABLED=0, STPMODE_STP=1, STPMODE_VLAN_STP=2} stp_mode_t;
typedef enum {STPSTATE_DISABLED=0, STPSTATE_LISTENING=1, STPSTATE_LEARNING=2,
              STPSTATE_FORWARDING=3, STPSTATE_BLOCKING=4} stp_state_t;
typedef enum {STPROLE_ROOT=0, STPROLE_DESIGNATED, STPROLE_ALTERNATE,
              STPROLE_BACKUP, STPROLE_DISABLED} stp_role_t;

typedef enum {LIBSW_OBJID_IFACE=1, LIBSW_OBJID_VLAN, LIBSW_OBJID_LAG,
              LIBSW_OBJID_VMT, LIBSW_OBJID_VIFACE} libsw_objid_t;

/**
 * Defines a number of possible filters for the
 * #libsw_get_vlan_members function. The default value of 0
 * corresponds to ANY member object type.
 */
typedef enum {
  LIBSW_VLAN_ANY = 0,           ///< No filtering
  LIBSW_VLAN_IFACE,             ///< Only physycal interfaces (ports)
  LIBSW_VLAN_LAG,               ///< Only aggregated interfaces (LAGs)
  LIBSW_VLAN_IFACE_LAG          ///< Only physycal interfaces that
                                ///< aren't the members of any LAG and LAGs                                 
} libsw_vlan_member_filter_t;


typedef struct libsw_iface_stp_status_s
{
  stp_role_t role;
  uint32_t cost;
  uint8_t priority;
  uint8_t portnum;
} libsw_iface_stp_status_t;

#define LIBSW_STP_IFACE_PARAM_FL_AUTOCOST 0x0001
#define LIBSW_STP_IFACE_PARAM_DEFAULT_PRIORITY (128)

typedef struct libsw_iface_stp_params_s
{
  uint16_t flags;
  uint32_t cost;
  uint8_t priority;
} libsw_iface_stp_params_t;

typedef struct stp_status_s
{
  uint8_t  rootid[6];
} libsw_stp_status_t;

typedef struct stp_params_s
{
  uint8_t  bridgeid[6];
  uint16_t priority;
  uint16_t max_age;
  uint16_t hello_time;
  uint16_t forward_delay;
  uint16_t hold_time;
} libsw_stp_params_t;

typedef struct libsw_iface_igmp_snooping_status_s
{
  uint8_t uplink;
} libsw_iface_igmp_snooping_status_t;

typedef struct libsw_iface_igmp_snooping_params_s
{
  uint8_t uplink;

  //array of statically assigned group MAC addresses
  //void *static_mac;
} libsw_iface_igmp_snooping_params_t;

typedef struct igmp_snooping_status_s
{
} libsw_igmp_snooping_status_t;

typedef struct igmp_snooping_params_s
{
  uint8_t report_flood; //send Report Messages to all interfaces rather than 
                        //uplinks only
  uint8_t max_version;

  uint32_t tablesize;
  uint32_t groupsize;

  // These three values are used to calculate one timer: "Group
  // Membership Interval": 
  uint8_t robustness; //times *
  int query_interval; //seconds +
  int response_interval; // 0.1 seconds

} libsw_igmp_snooping_params_t;

typedef struct {
  char name[LIBSW_OBJNAMESIZE];
  uint16_t num;
  uint8_t type;
  uint32_t flags;
  libsw_objid_t id;
} libsw_obj_t;

typedef struct
{
  char name[33];
  char version[128];
} hw_info_t;

typedef struct
{
  union {
    libsw_obj_t obj;
    struct {
      char name[LIBSW_OBJNAMESIZE];
      uint16_t num;
      uint8_t type;
      uint32_t flags;
    };
  };
} iface_desc_t;

typedef enum
{
  LIBSW_STATITEM_UNKNOWN=0,
  LIBSW_STATITEM_RX_PACKETS,
  LIBSW_STATITEM_TX_PACKETS,
  LIBSW_STATITEM_RX_BYTES,
  LIBSW_STATITEM_TX_BYTES,
  LIBSW_STATITEM_RX_DROP_PACKETS,
  LIBSW_STATITEM_TX_DROP_PACKETS
} stat_item_id_t;

#define LIBSW_STATITEM_FL_HIDDEN 1

typedef struct
{
  char *name;
  stat_item_id_t id;
  uint8_t flags;
  uint64_t value;
} stat_item_t;

typedef struct
{
  uint32_t flags;
} iface_status_t;

typedef struct vlan_desc
{
  union {
    libsw_obj_t obj;
    struct {
      char name[LIBSW_OBJNAMESIZE];
      uint16_t vid;
      uint8_t type;
      uint32_t flags;
    };
  };
} vlan_desc_t;

typedef struct
{
  vlan_desc_t  *vlan;
  uint8_t tag:1;
} iface_vlan_t;

typedef struct
{
  const libsw_obj_t *member;
  uint8_t tag : 1;
} vlan_member_t;

typedef struct
{
  char vendor[17];
  char partnum[17];
  float txpower;
  float rxpower;
  uint32_t bitrate;
  uint32_t spmode;
} sfp_info_t;

typedef struct {
  union {
    uint8_t mac[6];			/* MAC address */
    uint64_t mac_num;
  };
  vlan_desc_t *vlan;			/* VID 1-4094, 0 for SVL */
  iface_desc_t *iface;			/* */
  uint16_t flags;
} mac_table_t;

/**
 * \brief flags for MAC table items
*/
#define LIBSW_MACTABLE_FL_STATIC 1

typedef struct {
  union {
    libsw_obj_t obj;
    struct {
      char name[LIBSW_OBJNAMESIZE];
      uint16_t num;
    };
  };
} lag_desc_t;

typedef struct {
  union {
    libsw_obj_t obj;
    struct {
      char name[LIBSW_OBJNAMESIZE];
      uint16_t num;
    };
  };
} vmt_desc_t;

typedef struct {
  libsw_obj_t obj;
} viface_desc_t;

typedef struct {
  vlan_desc_t *src;
  vlan_desc_t *dst;
} vmt_item_t;

typedef struct {
  uint32_t speed;
  union {
    uint32_t flags;
    struct {
      uint8_t rs0:2;
      uint8_t rs1:2;
      uint8_t passive:1;
    };
  };
} speed_mode_t;

typedef enum  {
  ACL_COND_UNKNOWN=0,
  ACL_COND_DMAC,
  ACL_COND_SMAC,
  ACL_COND_DIP,
  ACL_COND_SIP,
  ACL_COND_ETYPE,
  ACL_COND_VID,
  ACL_COND_IP_PROTO,
  ACL_COND_SPORT,
  ACL_COND_DPORT,
  ACL_COND_DIP6,
  ACL_COND_SIP6,
  ACL_COND_IFACE,
  ACL_COND_MASK,
  ACL_COND_HEXDATA,
  ACL_COND_HEXOFFSET
} libsw_acl_condition_type_t;

#define ACL_COND_FLAG_NEG 1

typedef union libsw_acl_data_s {
  uint8_t mac[6];
  uint16_t iface_num;
  uint16_t vlan_id;
  uint8_t ip4addr[4];
  uint8_t ip6addr[16];
  uint8_t cos:3;
  uint8_t priority:6;
  uint8_t proto;
  struct  {
    uint16_t from;
    uint16_t to;
  } ports;
  uint16_t etype;
  uint8_t mask;
  uint16_t offset;
} libsw_acl_data_t;

typedef struct libsw_acl_condition_s {
  libsw_acl_condition_type_t type;
  uint8_t *data;
  /*libsw_acl_data_t data;*/
  uint8_t flags;
  uint16_t size;
} libsw_acl_condition_t;

typedef enum {
  ACL_ACTION_UNKNOWN=0,
  ACL_ACTION_DROP,
  ACL_ACTION_DROP_ON_CONGESTION,
  ACL_ACTION_FORWARD,
  ACL_ACTION_CHANGE_VID,
  ACL_ACTION_CHANGE_COS,
  ACL_ACTION_CHANGE_DSCP,
  ACL_ACTION_L3FORWARD,
} libsw_acl_action_type_t;

typedef struct libsw_acl_action_s {
  libsw_acl_action_type_t type;
  uint16_t size;
  uint8_t *data;
} libsw_acl_action_t;

typedef struct libsw_acl_condition_list_s {
  libsw_acl_condition_t *condition;
  uint16_t size;
  uint16_t count;
} libsw_acl_condition_list_t;

typedef struct libsw_acl_rule_s {
  libsw_acl_condition_list_t conditions;
  libsw_acl_action_t action;
} libsw_acl_rule_t;

typedef struct libsw_acl_rule_list_s {
  libsw_acl_rule_t *rules;
  uint16_t size;
  uint16_t count;
} libsw_acl_rule_list_t;

typedef struct libsw_ipaddr_s {
  uint8_t proto;
  union {
    union {
      uint8_t ip[4];
      uint32_t ip_u32;
    } ip4;
  };
} libsw_ipaddr_t;

typedef struct libsw_ipmask_s {
  uint8_t proto;
  union {
    struct {
      uint8_t prefix:1;
      union {
        uint8_t mask[4];
        uint32_t mask_u32;
        uint8_t prefix_len;
      };
    } ip4;
  };
} libsw_ipmask_t;

typedef struct libsw_ipiface_s {
  libsw_ipaddr_t addr;
  libsw_ipmask_t mask;
  uint8_t mac[6];
  vlan_desc_t *vlan;
} libsw_ipiface_t;

typedef struct libsw_iproute_s {
  libsw_ipaddr_t dst;
  libsw_ipmask_t mask;
  libsw_ipaddr_t gw;
} libsw_iproute_t;

typedef enum {
  LIBSW_SLPROT_RAW=0,
  LIBSW_SLPROT_IGMP
} libsw_slow_prot_t;

typedef struct {
  libsw_slow_prot_t protocol;
} libsw_slow_prot_id_t;

typedef struct {
  libsw_obj_t *source;
  iface_desc_t *iface;
  vlan_desc_t *vlan;
  void *data;
  size_t size;
} libsw_slow_prot_hdr_t;

typedef enum {
  LIBSW_CH_ACTION_ADD=0,
  LIBSW_CH_ACTION_DEL
} libsw_change_action_t;


#endif
