#ifndef LIBSWITCH_H
#define LIBSWITCH_H

void test(void);

#include <stdint.h>
#include <stdio.h>
#include "libsw_debug.h"

#include "libsw_types.h"
#include "libsw_driver.h"
#include "libsw_handle.h"
#include "libsw_api.h"

#include "libdb.h" // x10-db

typedef struct {
  FILE *file;
  uint8_t flags;
} libsw_sfpdetails_t;

#define LIBSW_ITERATOR_COND_RANGE  1
#define LIBSW_ITERATOR_COND_TYPE   2
#define LIBSW_ITERATOR_COND_FLAGS  4

#define LIBSW_MODULE_NAME_TEMPLATE "libx10-switch-%s.so"

typedef struct
{
  uint8_t type;
  libsw_objid_t id;
  int curnum;
  int lastnum;
  int inc;
  uint8_t types[8];
  uint32_t flags;
  uint32_t nflags;
} libsw_obj_itr_t;

/* library interface */
int libsw_init(libsw_t **lsw, char *hwname, int l);
int libsw_free(libsw_t *lsw);

/* Connect hardware */
int libsw_connect_hardware(libsw_t *);

/* Initialize hardware */
int libsw_init_hardware(libsw_t *);
int libsw_reset_hardware(libsw_t *, uint16_t level);
int libsw_get_hw_info(libsw_t *, hw_info_t *info);

/* Object interface */
libsw_objid_t libsw_cli_get_objid(libsw_t *lsw, const char *name);
libsw_obj_t *libsw_get_obj_by_name(libsw_t *,libsw_objid_t id, const char *name);
libsw_obj_t *libsw_get_obj_by_num(libsw_t *,libsw_objid_t id, uint16_t num);
libsw_obj_t *libsw_get_obj_by_index(libsw_t *,libsw_objid_t id, uint16_t idx);
libsw_obj_t *libsw_cli_get_obj_by_name(libsw_t *,libsw_objid_t id, const char *name);
uint16_t libsw_get_obj_index(libsw_t *lsw, const libsw_obj_t *obj);
uint16_t libsw_get_obj_count(libsw_t *lsw,libsw_objid_t id);
int libsw_set_obj_name(libsw_t *lsw, libsw_obj_t* obj, const char *name);
int libsw_set_obj_hidden(libsw_t *lsw, libsw_obj_t* obj, uint8_t hidden);

/* Statistics */
int libsw_get_obj_stat(libsw_t *lsw, const libsw_obj_t *obj, stat_item_t *i, uint16_t *cnt);
int libsw_clear_obj_stat(libsw_t *lsw, const libsw_obj_t *obj);
stat_item_t *libsw_get_stat_item(stat_item_t *sitems, uint16_t cnt, stat_item_id_t id);

/* Iterators */
libsw_obj_itr_t *libsw_create_itr(libsw_t *);
void libsw_init_itr(libsw_t *, libsw_obj_itr_t *it, libsw_objid_t id);
void libsw_free_itr(libsw_t *, libsw_obj_itr_t *it);
int libsw_cli_add_itr_range(libsw_t *, libsw_obj_itr_t *it, const char *range);
int libsw_add_itr_type(libsw_t *, libsw_obj_itr_t *it, uint8_t types);
int libsw_set_itr_types(libsw_t *, libsw_obj_itr_t *it, uint8_t *types);
int libsw_add_itr_flags(libsw_t *, libsw_obj_itr_t *it, uint32_t flags, uint32_t nflags);
libsw_obj_t *libsw_get_next_obj(libsw_t *, libsw_obj_itr_t *it);

/* Interfaces */
int libsw_get_iface_status(libsw_t *, const iface_desc_t *, iface_status_t *);
int libsw_get_obj_status(libsw_t *, const libsw_obj_t *, iface_status_t *);
int libsw_set_iface_enable(libsw_t *, const iface_desc_t *, uint8_t enable);
int libsw_set_obj_enable(libsw_t *, const libsw_obj_t *, uint8_t enable);
int libsw_set_iface_loopback(libsw_t *, const iface_desc_t *, uint8_t enable);
int libsw_get_iface_dynamic_learning(libsw_t *lsw, const iface_desc_t *iface, uint8_t *enable);
int libsw_set_iface_dynamic_learning(libsw_t *lsw, const iface_desc_t *iface, uint8_t enable);
int libsw_get_obj_dynamic_learning(libsw_t *lsw, const libsw_obj_t *iface, uint8_t *enable);
int libsw_set_obj_dynamic_learning(libsw_t *lsw, const libsw_obj_t *iface, uint8_t enable);
int libsw_get_iface_portsecurity(libsw_t *lsw, const iface_desc_t *iface, iface_portsec_t *);
int libsw_set_iface_portsecurity(libsw_t *lsw, const iface_desc_t *iface, uint8_t value);

/* Interfaces VLANs */
int libsw_get_iface_defaultvlan(libsw_t *, const iface_desc_t *, vlan_desc_t **vlan);
int libsw_set_iface_defaultvlan(libsw_t *, const iface_desc_t *, vlan_desc_t * vlan);
int libsw_get_obj_defaultvlan(libsw_t *, const libsw_obj_t *, vlan_desc_t **vlan);
int libsw_set_obj_defaultvlan(libsw_t *, const libsw_obj_t *, vlan_desc_t * vlan);
int libsw_get_iface_vlans(libsw_t *, const iface_desc_t *, iface_vlan_t *, uint16_t *count);
int libsw_get_obj_vlans(libsw_t *, const libsw_obj_t *, iface_vlan_t *, uint16_t *count);
int libsw_add_iface_vlan(libsw_t *, const iface_desc_t *, iface_vlan_t *);
int libsw_del_iface_vlan(libsw_t *, const iface_desc_t *, vlan_desc_t * vlan);
int libsw_add_obj_vlan(libsw_t *, const libsw_obj_t *, iface_vlan_t *);
int libsw_del_obj_vlan(libsw_t *, const libsw_obj_t *, vlan_desc_t * vlan);
int libsw_get_vlan_members( libsw_t *lsw, const vlan_desc_t *vlan,
                            vlan_member_t *members, uint16_t *count,
                            libsw_vlan_member_filter_t filter );

/* VLAN mapping tables */
int libsw_set_iface_vmt(libsw_t *, const iface_desc_t *iface, vmt_desc_t *);
int libsw_get_iface_vmt(libsw_t *, const iface_desc_t *iface, vmt_desc_t **);
int libsw_get_vmt(libsw_t *, const vmt_desc_t *, vmt_item_t *, uint16_t *count);
int libsw_add_vmt_item(libsw_t *, const vmt_desc_t *, vmt_item_t *item);
int libsw_del_vmt_item(libsw_t *, const vmt_desc_t *, vmt_item_t *item);

/* Periodical update */
uint16_t get_update_time(libsw_t *);
int libsw_update(libsw_t *);

int libsw_get_iface_speed_mode(libsw_t *, const iface_desc_t *, speed_mode_t *mode);
int libsw_get_obj_speed_mode(libsw_t *, const libsw_obj_t *, speed_mode_t *mode);
int libsw_set_iface_speed_mode(libsw_t *, const iface_desc_t *, speed_mode_t *mode);

/* Interfaces link aggregation */
int libsw_get_iface_lag_state(libsw_t *, const iface_desc_t *, uint8_t *enable);
int libsw_set_iface_lag_state(libsw_t *, const iface_desc_t *, uint8_t enable);
int libsw_get_iface_lag(libsw_t *, const iface_desc_t *, lag_desc_t **lag);
int libsw_set_iface_lag(libsw_t *, const iface_desc_t *, const lag_desc_t *lag);
int libsw_get_lag_distribution(libsw_t *, const lag_desc_t *lag, uint16_t *mode);
int libsw_set_lag_distribution(libsw_t *, const lag_desc_t *lag, uint16_t mode);
int libsw_get_lag_members(libsw_t *, const lag_desc_t *lag, iface_desc_t *ifs[], uint16_t *count);

/* Interfaces mirroring */
int libsw_get_portmirror(libsw_t *, const iface_desc_t *, iface_desc_t **);
int libsw_set_portmirror(libsw_t *, const iface_desc_t *, const iface_desc_t *);

/* VLANs mirroring */
int libsw_get_vlanmirror(libsw_t *, const vlan_desc_t *, iface_desc_t **);
int libsw_set_vlanmirror(libsw_t *, const vlan_desc_t *, const iface_desc_t *);

/* SFP */
int libsw_get_sfp_info(libsw_t *, const iface_desc_t *, sfp_info_t *);
int libsw_show_sfp_details(libsw_t *, const iface_desc_t *, libsw_sfpdetails_t *);
int libsw_get_sfp_speed_mode(libsw_t *, const iface_desc_t *, speed_mode_t *smode);
int libsw_handle_sfp_after_install(libsw_t *, const iface_desc_t *);

/* MAC Table */
int libsw_get_mac_table(libsw_t *, mac_table_t *, uint32_t *count);
int libsw_add_mac_table_entry(libsw_t *, const mac_table_t *);
int libsw_del_mac_table_entry(libsw_t *, const mac_table_t *);

/* STP */

int libsw_set_stp(libsw_t *lsw, uint8_t en);
int libsw_get_stp(libsw_t *lsw, uint8_t *en);
int libsw_set_stp_mode(libsw_t *lsw, stp_mode_t stpmode);
int libsw_get_stp_mode(libsw_t *lsw, stp_mode_t *stpmode);
int libsw_set_iface_stp(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, uint8_t en);
int libsw_get_iface_stp(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, uint8_t *en);
int libsw_set_obj_stp(libsw_t *lsw, const libsw_obj_t *viface, const vlan_desc_t *vlan, uint8_t en);
int libsw_get_obj_stp(libsw_t *lsw, const libsw_obj_t *viface, const vlan_desc_t *vlan, uint8_t *en);
int libsw_set_iface_stp_state(libsw_t *lsw, const iface_desc_t *iface, vlan_desc_t *vlan, stp_state_t stpstate);
int libsw_get_iface_stp_state(libsw_t *lsw, const iface_desc_t *iface, vlan_desc_t *vlan, stp_state_t *stpstate);
int libsw_set_obj_stp_state(libsw_t *lsw, const libsw_obj_t *viface, vlan_desc_t *vlan, stp_state_t stpstate);
int libsw_get_obj_stp_state(libsw_t *lsw, const libsw_obj_t *viface, vlan_desc_t *vlan, stp_state_t *stpstate);

int libsw_set_iface_stp_params(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, const libsw_iface_stp_params_t *p);
int libsw_get_iface_stp_params(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, libsw_iface_stp_params_t *p);
int libsw_set_obj_stp_params(libsw_t *lsw, const libsw_obj_t *viface, const vlan_desc_t *vlan, const libsw_iface_stp_params_t *p);
int libsw_get_obj_stp_params(libsw_t *lsw, const libsw_obj_t *viface, const vlan_desc_t *vlan, libsw_iface_stp_params_t *p);

int libsw_set_stp_params(libsw_t *lsw, const vlan_desc_t *vlan, const libsw_stp_params_t *p);
int libsw_get_stp_params(libsw_t *lsw, const vlan_desc_t *vlan, libsw_stp_params_t *p);
int libsw_get_stp_status(libsw_t *lsw, const vlan_desc_t *vlan, libsw_stp_status_t *p);
int libsw_set_stp_status(libsw_t *lsw, const vlan_desc_t *vlan, const libsw_stp_status_t *p);

int libsw_get_iface_stp_status(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, libsw_iface_stp_status_t *p);
int libsw_set_iface_stp_status(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, const libsw_iface_stp_status_t *p);
int libsw_get_obj_stp_status(libsw_t *lsw, const libsw_obj_t *viface, const vlan_desc_t *vlan, libsw_iface_stp_status_t *p);
int libsw_set_obj_stp_status(libsw_t *lsw, const libsw_obj_t *viface, const vlan_desc_t *vlan, const libsw_iface_stp_status_t *p);

/* IGMP*/
int libsw_set_igmp_snooping_enable(libsw_t *lsw, const vlan_desc_t *vlan, uint8_t enable);
int libsw_get_igmp_snooping_enable(libsw_t *lsw, const vlan_desc_t *vlan, uint8_t *enable);

int libsw_set_iface_igmp_snooping_enable(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, uint8_t en);
int libsw_get_iface_igmp_snooping_enable(libsw_t *lsw, const iface_desc_t *iface, const vlan_desc_t *vlan, uint8_t *en);
int libsw_set_obj_igmp_snooping_enable(libsw_t *lsw, const libsw_obj_t *obj, const vlan_desc_t *vlan, uint8_t en);
int libsw_get_obj_igmp_snooping_enable(libsw_t *lsw, const libsw_obj_t *obj, const vlan_desc_t *vlan, uint8_t *en);

int libsw_set_igmp_snooping_params(libsw_t *lsw, const vlan_desc_t *vlan,
                                   const libsw_igmp_snooping_params_t *prms);
int libsw_get_igmp_snooping_params(libsw_t *lsw, const vlan_desc_t *vlan, 
                                   libsw_igmp_snooping_params_t *prms);
int libsw_set_iface_igmp_snooping_params(libsw_t *lsw, const iface_desc_t *iface, 
                                         vlan_desc_t *vlan,
                                         const libsw_iface_igmp_snooping_params_t *prm);
int libsw_get_iface_igmp_snooping_params(libsw_t *lsw, const iface_desc_t *iface, 
                                         const vlan_desc_t *vlan, 
                                         libsw_iface_igmp_snooping_params_t *prm);
int libsw_set_obj_igmp_snooping_params(libsw_t *lsw, const libsw_obj_t *obj,
                                         vlan_desc_t *vlan,
                                         const libsw_iface_igmp_snooping_params_t *prm);
int libsw_get_obj_igmp_snooping_params(libsw_t *lsw, const libsw_obj_t *obj,
                                         const vlan_desc_t *vlan,
                                         libsw_iface_igmp_snooping_params_t *prm);
/* ACL */
int libsw_acl_init_condition_list(libsw_acl_condition_list_t *rl, uint16_t maxsize);
int libsw_acl_free_condition_list(libsw_acl_condition_list_t *rl);
int libsw_acl_add2condition_list(libsw_acl_condition_list_t *rl, libsw_acl_condition_type_t ruletype,
                                 void *data, uint16_t size, uint8_t flags);

int libsw_acl_init_rule_list(libsw_acl_rule_list_t *rs, uint16_t maxsize);
int libsw_acl_free_rule_list(libsw_acl_rule_list_t *rs);

int libsw_acl_add_rule(libsw_t *lsw, libsw_acl_condition_list_t *rl, libsw_acl_action_t *act);
int libsw_acl_del_rule(libsw_t *lsw, libsw_acl_condition_list_t *rl, libsw_acl_action_t *act);
int libsw_acl_del_rule_by_id(libsw_t *lsw, uint16_t id);
int libsw_acl_get_rules(libsw_t *lsw, libsw_acl_rule_list_t *rsl);
int libsw_acl_clear_all(libsw_t *lsw);

int libsw_set_mac_aging_time(libsw_t *lsw, uint32_t time);
int libsw_get_mac_aging_time(libsw_t *lsw, uint32_t *time);

/* IP interfaces */
int libsw_add_ip_iface(libsw_t *lsw, const libsw_ipiface_t *ipif);
int libsw_del_ip_iface(libsw_t *lsw, const libsw_ipiface_t *ipif);
int libsw_get_ip_ifaces(libsw_t *lsw, libsw_ipiface_t *ifs, uint16_t *count);

/* IP routes */
int libsw_add_ip_route(libsw_t *lsw, const vlan_desc_t *vlan, const libsw_iproute_t *rt);
int libsw_del_ip_route(libsw_t *lsw, const vlan_desc_t *vlan, const libsw_iproute_t *rt);
int libsw_get_ip_routes(libsw_t *lsw, const vlan_desc_t *vlan, libsw_iproute_t *rts, uint16_t *count);

int libsw_get_l4_proto_num(char *pname, uint8_t *pnum);
int libsw_get_l4_proto_name(uint8_t pnum, char *pname, uint8_t plen);
int libsw_get_l4_serv_port(char *sname, uint16_t *pnum);
int libsw_get_l4_serv_name(uint16_t pnum, char *sname, uint8_t l);

/* Packet send/receive */
int libsw_open_slow_protocol_fd(libsw_t *lsw, int *fd, libsw_slow_prot_id_t *slp);
int libsw_read_slow_protocol_packet(libsw_t *lsw, int fd, uint8_t flags, char *buf, size_t *size, libsw_slow_prot_hdr_t *hdr);
int libsw_prepare_packet_for_send( libsw_t *lsw,
                                   const libsw_obj_t *obj,
                                   const vlan_desc_t *vlan,
                                   void *data,
                                   size_t *len,
                                   size_t bufsize );
int libsw_send_packet( libsw_t *h,
                       const libsw_obj_t *obj,
                       const void *prepared_data,
                       size_t size );

/* Jumbo frames support */
int libsw_check_jumbo_available( libsw_t *lsw );
int libsw_set_management_configuration_ready( libsw_t *lsw, char ready );

int libsw_set_port_jumbo_frame_length( libsw_t *lsw, const iface_desc_t *iface, uint16_t len);
int libsw_port_disable_jumbo_frames_support( libsw_t *lsw, const iface_desc_t *iface );
int libsw_get_iface_frame_length( libsw_t *lsw, const iface_desc_t *iface, uint16_t *len );

void libsw_set_debug(libsw_t *, int level, FILE *f);
char *libsw_strerror(int code);


#endif

